#!/usr/bin/perl -I /usr/local/lib/perl/site_perl/
# An algorithm for managing zfs snapshots

use strict;
use warnings;
use List::Util qw[min max];
use Data::Dumper;

use ZFS::ZFS;
use ZFS::Config;
use ZFS::Host;

my $config=ZFS::Config->new("/etc/local/zfsbackup-cfg.pl");

my $pair=ZFS::ZFS->new("ch-fileserver.ch.private.cam.ac.uk", "pennington.ch.private.cam.ac.uk");

$pair->zfsName("data/group/atm");
my $host;

my @children=$pair->getChildFilesystems;
my $list={};
my $mostrecent;
foreach my $child (@children) {
	
	print $child." - ";
	print scalar(localtime(time)). " to.. ";
	$pair->updateCommonBackupTag;
	
	$list={};

	$pair->zfsName($child);

 	$host=$pair->srcHost;
	$list=$host->getListForPrune;
	$mostrecent=$host->getLastSnapTime;
	&addtags($list);

	$host=$pair->destHost;
	$list=$host->getListForPrune;
	$mostrecent=$host->getLastSnapTime;
	&addtags($list);
	print scalar(localtime(time))."\n";
}

exit;

# apply "commonbackup" tag to most recent common snapshot
# remove "commonbackup" tag from any other snapshot
# generate $list 
# add tags to $list
# delete anything that's untagged


sub detail($)  { 
	my ($list)=@_;
	foreach my $k (sort(keys(%{$list}))) {
		print "$k: ".scalar(localtime($k)).' '.join(', ',@{$list->{$k}})."\n";
	}

}


sub summary($) {
	my $list=shift;
	foreach my $type ($config->getArchiveTypes) {
		print "$type: ".scalar(grep {grep /$type/,@{$list->{$_}};} keys(%{$list}))."\n";
	}
	print "Total: ".scalar(keys(%{$list}))."\n";
}


sub addtags($) {
	#my $list=shift;
	# Add tags for each type
	foreach my $type ($config->getArchiveTypes) {
		&addtag($type);
	}
	&deleteuntagged();
}

sub addtag($) {
	my ($type)=@_;
	#print "Investigating $type tags\n";
	#print Dumper($list);
	#print "Most recent: $mostrecent\n";
	my $maxage=$mostrecent-$config->ageOfArchiveType($type);
	my $step=$config->ageOfArchiveType($type)/$config->numberOfArchiveType($type);
	my $c=&counttags($type,$list);
	# Bootstrap code
	if ($c==0) {
		# Find oldest snapshot < maximum age
		#print "Bootstrapping $type: need candidates with stamp>$maxage\n";
		my @candidates=sort(grep {$_>=($maxage)} keys(%{$list}));
		my $bootstrap=$candidates[0];
		push(@{$list->{$bootstrap}},$type);
		$host->holdSnapshot($type,$bootstrap);
		#print "Bootstrapping $type by tagging $bootstrap\n";
		#print Dumper($list);
	}
	# Algorithm code: Find newest tagged snapshot
	my $leftmost=&newest($type,$list);
	#print "Located $leftmost tagged as $type \n";
	# Find any snapshots to add
	#print "Seeking others newer than ".($leftmost+$step)." $step away\n";
	my $nextsnap=&newerthan($leftmost+$step,$list);
	if ($nextsnap) { 
		#print "Found $nextsnap - tagging\n";
		push(@{$list->{$nextsnap}},$type);
		$host->holdSnapshot($type,$nextsnap);
		#print Dumper($list);
		&addtag($type);
	}
	# De-tag any snapshots which are older than $maxage
	my @candidates=grep {$_<$maxage} keys(%{$list});
	foreach my $k (sort(@candidates)) {
		my $c=scalar(@{$list->{$k}});
		#print "$k has $c entries\n";
		if (scalar(@{$list->{$k}})>0) {
			$host->releaseSnapshot($type,$k) if grep(/$type/, @{$list->{$k}});
#print "removing $type from $k : ".join(', ',@{$list->{$k}})."\n";
			my @l=grep {!/$type/} @{$list->{$k}};
			$list->{$k}=[];
			push(@{$list->{$k}},@l);
#print "removed $type from $k : ".join(', ',@{$list->{$k}})."\n";
		}
	}
}

sub deleteuntagged() {
# Remove any untagged snapshots
	foreach my $k (keys(%{$list})) {
		if (scalar(@{$list->{$k}})==0) {
			delete $list->{$k};
			$host->destroySnapshot($k);
		}
	}
}

sub newerthan($$) {
	my ($age,$list)=@_;
	my @list=(sort(grep {$_>=$age} keys(%{$list})));

	return $list[0];
}

sub newest($$) {
	my ($type,$list)=@_;
	my @list=reverse(sort(grep { grep /$type/,@{$list->{$_}};} keys(%{$list})));
	return $list[0];
}

sub counttags($$) {
	my ($type,$list)=@_;
	#my $count=0;
	my $count=scalar(grep { grep /$type/,@{$list->{$_}};} keys(%{$list}));
	#print "Found $count $type items\n";
	return $count;
}

sub printsummary($) {
}
