#!/usr/bin/perl -I /usr/local/lib/perl/site_perl


use ZFS::Host;
use Data::Dumper;
use ZFS::Config;
zfstidyup('zpool1/ldap-3.ch.cam.ac.uk/zfs-rsnap');
exit;

sub zfstidyup($) {
 my $zfs=shift;
 my $zfshost=new ZFS::Host('localhost');
 print "Tidying up snapshots on $zfs\n";
 $arr = {
          'weekly' => {
                        'count' => 4,
                        'flagname' => 'weekly',
                        'age' => 2419200,
                        'backup_log_id' => 21030
                      },
          'hourly' => {
                        'count' => 6,
                        'flagname' => 'hourly',
                        'age' => 86400,
                        'backup_log_id' => 21030
                      },
          'monthly' => {
                         'count' => 6,
                         'flagname' => 'monthly',
                         'age' => 604800,
                         'backup_log_id' => 21030
                       },
          'daily' => {
                       'count' => 7,
                       'flagname' => 'daily',
                       'age' => 604800,
                       'backup_log_id' => 21030
                     }
        };

 my %archivepolicy=%$arr;
 my $config=ZFS::Config->void();
 $config->archivepolicy($arr);
 
 $zfshost->zfsName($zfs);
 my $list=$zfshost->getListForPrune();
 #print "Currently have \n";
 #print Dumper($list);
 my $mostrecent=$zfshost->getLastSnapTime();
print "Most recent is $mostrecent\n";
 my $olist=$list;
 foreach my $type ($config->getArchiveTypes) {
  #print "Preparing $type pass\n";
  #print "Config is:\n";
  #print Dumper($config);
  $list=&addtag($type,$mostrecent,\$config,\$list,\$zfshost);
  #print "Finished $type pass\n";
  #print Dumper($list);
 }
print "New List\n";
print Dumper($list);
print "Old List\n";
print Dumper($olist);
#print "ZFShost\n";
#print Dumper($zfshost);
#print "Calling delete untagged\n";
 print "Removing untagged snapshots:\n";
 print Dumper($list);
 #&deleteuntagged(\$list,\$zfshost);


 #print Dumper($list);
 #use Data::Dumper;
 #print Dumper($zfshost);
 print "Tidied\n";
}

# Indicate when a host has joined a queue. The returned ID identifies this backup job
sub stampjoinqueue($$) {
 my ($dbh,$id)=@_;
 my $sql="insert into backup_log (backup_task_id, joined_queue) values (?,now()) returning backup_log_id";
 my $sth=$dbh->prepare($sql);
 my $rv=$sth->execute($id);
 if (! $rv) {
  print "Odd, failed to insert into backup_log\n";
  exit;
 }
 my $rec=$sth->fetchrow_hashref();
 return $rec->{'backup_log_id'};
 #return $a[0];
}

# Indicate when a backup started
sub stampbackupstart($$) {
 my ($dbh,$id)=@_;
 my $sql="update backup_log set started_processing=now() where backup_log_id=?";
 $dbh->do($sql,undef,$id);
}

# Indicate when a backup has ended
sub stampbackupstop($$$) {
 my ($dbh,$id,$exit)=@_;
 my $sql="update backup_log set ended_processing=now(), exit_code=? where backup_log_id=?";
 $dbh->do($sql,undef,$exit,$id);
}

sub rsnapshotextrainfo($$$) {
 my ($dbh,$id,$type)=@_;
 print "Updating backup_log_rsnapshot backup_log_id=$id, type=$type\n";
 my $sql="insert into backup_log_rsnapshot (backup_log_id,rsnapshot_subtype_id) values (?,(select id from rsnapshot_subtypes where desc=?))"; 
 $dbh->do($sql,undef,$id,$type);
}

# Remove incomplete log entries
sub tidydatabase() {
 my $dbh=DBI->connect("dbi:Pg:dbname=$dbfile",$dbuser,$dbpass);
 #my $sql='delete from backup_log where offline !=1 or offline is null and ended_processing is null';
 my $sql='delete from backup_log where ended_processing is null and ((not offline_once) or offline_once is null)';
 $dbh->do($sql,undef);
}

sub setoffline($$) {
 my ($dbh,$id)=@_;
 my $sql='update backup_log set offline=1 where backup_log_id=?';
 $dbh->do($sql,undef,$id);
}

# Examine control queue
sub examinecontrolq($) {
 my $dmask=shift;
 my $return=0;
 {
  lock($controlq);
  if ($controlq->pending()) {
   my $msg=$controlq->peek();
   if (($msg & D_MASK) == $dmask) {
    $msg=$controlq->dequeue();
   }
   $return=($msg & M_MASK);
  }
 }
 return $return;
}

sub getzpools() {
 my @zpools=`zpool list -H -o name`;
 chomp(@zpools);
 return @zpools;
}

sub addtag($$$$$) {
        #my ($type,$mostrecent,$config,$list,$host)=@_;
        my $type=shift;
	my $mostrecent=shift;
	my $rconfig=shift;
	my $rlist=shift;
	my $rhost=shift;
	my $config=$$rconfig;
	my $list=$$rlist;
	my $host=$$rhost;
print "Most recent is $mostrecent\n";
print "Maximum age of $type is ".$config->ageOfArchiveType($type)."\n";
        my $maxage=$mostrecent-($config->ageOfArchiveType($type));
print "Which I calculate to be $maxage\n";
        my $step=$config->ageOfArchiveType($type)/$config->numberOfArchiveType($type);
        my $c=&counttags($type,$list);
        # Bootstrap code
        if ($c==0) {
                # Find oldest snapshot < maximum age
                #print "Bootstrapping $type: need candidates with stamp>$maxage\n";
                my @candidates=sort(grep {$_>=($maxage)} keys(%{$list}));
                my $bootstrap=$candidates[0];
                push(@{$list->{$bootstrap}},$type);
                $host->holdSnapshot($type,$bootstrap);
                print "Bootstrapping $type by tagging $bootstrap\n";
                #print Dumper($list);
        }
        # Algorithm code: Find newest tagged snapshot
        my $leftmost=&newest($type,$list);
        print "Located $leftmost as most recent $type \n";
        # Find any snapshots to add
        print "Seeking with stamps greater than ".($leftmost+$step)." - $step away\n";
        ##my $nextsnap=&newerthan($leftmost+$step,$list);
        while ($nextsnap=&newerthan($leftmost+$step,$list)) { 
                print "Found $nextsnap - tagging\n";
                push(@{$list->{$nextsnap}},$type);
                $host->holdSnapshot($type,$nextsnap);
		$leftmost=$nextsnap;
        }
        # De-tag any snapshots which are older than $maxage
print "Max age is $maxage\n";
        my @candidates=grep {$_<$maxage} keys(%{$list});
        foreach my $k (sort(@candidates)) {
                my $c=scalar(@{$list->{$k}});
                #print "$k has $c entries\n";
                if (scalar(@{$list->{$k}})>0) {
                        $host->releaseSnapshot($type,$k) if grep(/$type/, @{$list->{$k}});
print "removing $type from $k : ".join(', ',@{$list->{$k}})."\n";
                        my @l=grep {!/$type/} @{$list->{$k}};
                        $list->{$k}=[];
                        push(@{$list->{$k}},@l);
print "removed $type from $k : ".join(', ',@{$list->{$k}})."\n";
                }
        }
print "After $type list is\n";
#print Dumper($list);
 foreach my $k (sort(keys(%{$list}))) {
 print "$k ".join(', ',@{$list->{$k}})."\n";
 }
	return $list;
}

sub deleteuntagged($$) {
	#my ($list,$host)=shift;
	my $rlist=shift;
	my $list=$$rlist;
	my $rhost=shift;
	my $host=$$rhost;
#print "In deleteuntagged : list\n";
#print Dumper($list);
#print "--HOST---\n";
#print Dumper($host);
# Remove any untagged snapshots
        foreach my $k (keys(%{$list})) {
                if (scalar(@{$list->{$k}})==0) {
                        delete $list->{$k};
                        $host->destroySnapshot($k);
                }
        }
}


sub newerthan($$) {
        my ($age,$list)=@_;
        my @list=(sort(grep {$_>=$age} keys(%{$list})));

        return $list[0];
}

sub newest($$) {
        my ($type,$list)=@_;
        my @list=reverse(sort(grep { grep /$type/,@{$list->{$_}};} keys(%{$list})));
        return $list[0];
}

sub counttags($$) {
        my ($type,$list)=@_;
        #my $count=0;
        my $count=scalar(grep { grep /$type/,@{$list->{$_}};} keys(%{$list}));
        #print "Found $count $type items\n";
        return $count;
}

