#!/bin/bash
#data to zpool1/chm.ch.cam.ac.uk/data
set -xv
HN=$1
SRC=$2
TGT=$3
REC=$4
if [ -z $3 ] ; then
 echo Use: $0 Hostname SourceZFS TargetZFS Recursive
 echo Where Recursive=1 for recursion
 exit 2
fi
MBUFARGS="-m 256M -s 128k -q "
LOGDIR=/var/log/backups/

SSH="ssh"

if [ -f /etc/zfs-rsync.d/${HN}_ZFS_${SRC//\//.} ] ; then
 # This file can define PRE and POST as scripts which are run
 echo Including settings from /etc/zfs-rsync.d/${HN}_ZFS_${SRC//\//.}
 . /etc/zfs-rsync.d/${HN}_ZFS_${SRC//\//.}
fi

set -e
if [ -t 1 ] ; then 
set -xv
fi

function latestcommonsnapshot {
 #REMOTE=`tempfile`
 #LOCAL=`tempfile`
 REMOTE=`mktemp -t zfs-REMOTE`
 LOCAL=`mktemp -t zfs-LOCAL`
 if $SSH $HN zfs list $SRC >/dev/null 2>&1 ; then
  $SSH $HN zfs list -t all -oname -r $SRC | grep $SRC@ | sed s/.*@// | grep ^[0-9]*$ >$REMOTE
 fi
 if zfs list $TGT >/dev/null 2>&1 ; then
  zfs list -t all -oname -r $TGT | grep $TGT@ | sed s/.*@// | grep ^[0-9]*$ >$LOCAL
 fi
 comm -12 $REMOTE $LOCAL | sort -rn | head -1
 rm $REMOTE $LOCAL
}

function findchildren {
 ZFS=$1
 $SSH $HN zfs list -r -t filesystem -o name -H $ZFS | grep -v ^$ZFS$
}

(
set +e
# If recursive, do all that for child ZFSes too
if [ z"$REC" = z1 ] ; then
 ZFSES=`findchildren $SRC`
 for Z in $ZFSES ; do
  # Call ourselves again
  if ! (echo ,$ZFSOMIT, | grep -q ,$Z,) ; then
   $0 $HN $Z $TGT${Z/$SRC/} 0
  fi
 done
fi
set -e

# Find the latest common snapshot
COM=`latestcommonsnapshot`
echo Most recent snapshot is $COM

echo OK | $SSH $HN mbuffer -q >/dev/null 2>/dev/null
NOMBUFFER=$?
HOLD=backup-`hostname -s`
TIME=`date +%s`
$SSH $HN zfs snapshot $SRC@$TIME
echo running $SSH $HN zfs snapshot $SRC@$TIME
$SSH $HN zfs hold $HOLD $SRC@$TIME
echo running $SSH $HN zfs hold $HOLD $SRC@$TIME
if [ -z $COM ] ; then
 # Enter bootstrap routine
 echo Bootstrap
 if [ $NOMBUFFER = 0 ] ; then
  $SSH $HN "zfs send $SRC@$TIME | mbuffer $MBUFARGS" | mbuffer $MBUFARGS 2>/dev/null| zfs receive -F $TGT
 else
  $SSH $HN "zfs send $SRC@$TIME" | zfs receive -F $TGT
 fi
 # Set hourly hold on this snapshot
 zfs hold hourly $TGT@$TIME
else
 # Have mbuffer?
 if [ $NOMBUFFER = 0 ] ; then
  # Just trying to figure out why this is not backing up farish...hence lots of echos
  $SSH $HN "zfs send -i $COM $SRC@$TIME | mbuffer $MBUFARGS" | mbuffer $MBUFARGS 2>/dev/null | zfs receive -F $TGT
  echo $HN pipeline returned "${PIPESTATUS[@]}"
 else
  $SSH $HN "zfs send -i $COM $SRC@$TIME" | zfs receive -F $TGT
  echo $HN pipeline returned "${PIPESTATUS[@]}"
 fi
 # Remove this hold from the previous common snapshot
 # but don't worry about error
 echo running $SSH $HN "zfs release $HOLD $SRC@$COM" 
 $SSH $HN "zfs release $HOLD $SRC@$COM" || true
fi
) >$LOGDIR/$HN 2>&1
